@@ -11,88 +11,98 @@
 #      modify it under the terms of Perl's Artistic License.        #
 #                                                                   #
 #####################################################################
-#
-# Net-IRC 0.71
-# ------------
-# 
-# Pete Sergeant's Changelog
-# 
-# July 1st 2001:
-#	=> Added some extra numeric events from Austnet, thanks to
-#		Andrew Macks (andypoo).
 
+# there used to be lots of cute little log quotes from #perl in here
+#
+# they're gone now because they made working on this already crappy
+# code even more annoying... 'HI!!! I'm from #perl and so I don't
+# write understandable, maintainable code!!! You see, i'm a perl
+# badass, so I try to be as obscure as possible in everything I do!'
+#
+# Well, welcome to the real world, guys, where code needs to be
+# maintainable and sane.
 
 package Net::IRC::Event;
 
 use strict;
-my %_names;
-
-# -- #perl was here! --
-# <fimmtiu> OK, another mini-log has been added to the mjd quote file
-#           in Event.pm. :-)
-# <amagosa> Heh heh
-#    <\mjd> There's an entire MJD quote file?
-#    <\mjd> Is it appropriate to ask you to put in the URL of my Perl
-#           Paraphernalia page?
-#    <\mjd> People who are amused might want to come visit.
-# <fimmtiu> Sure, why not... what's the url?
-#    <\mjd> http://perl.plover.com/
+our %_names;
 
+# Constructor method for Net::IRC::Event objects.
+# Takes at least 4 args:  the type of event
+#                         the person or server that initiated the event
+#                         the recipient(s) of the event, as arrayref or scalar
+#                         the name of the format string for the event
+#            (optional)   any number of arguments provided by the event
+sub new {
+  my $class = shift;
+  my $type = shift;
+  my $from = shift;
+  my $to = shift;
+  my $format = shift;
+  my $args = \@_;
+  
+  my $self = {
+    'type'   =>  $type,
+    'from'   =>  undef,
+    'to'     =>  ref($to) eq 'ARRAY'  ?  $to  :  [ $to ],
+    'format' =>  $format,
+    'args'   =>  [],
+  };
+  
+  bless $self, $class;
+  
+  if ($self->type !~ /\D/) {
+    $self->type($self->trans($self->type));
+  } else {
+    $self->type(lc($self->type));
+  }
+  
+  $self->from($from);     # sets nick, user, and host
+  $self->args($args);  # strips colons from args
+  
+  return $self;
+}
 
 # Sets or returns an argument list for this event.
 # Takes any number of args:  the arguments for the event.
 sub args {
-    my $self = shift;
+  my $self = shift;
+  my $args = shift;
 
-    if (@_) {
-        my (@q, $i, $ct) = @_;       # This line is solemnly dedicated to \mjd.
+  if($args) {  
+    my (@q, $i, $ct) = @{$args};       # This line is solemnly dedicated to \mjd.
 
-        $self->{'args'} = [ ];
-        while (@q) {
-            $i = shift @q;
-            next unless defined $i;
+    $self->{'args'} = [ ];
+    while (@q) {
+      $i = shift @q;
+      next unless defined $i;
 
-            if ($i =~ /^:/ and $ct) {                # Concatenate :-args.
-                 $i = join ' ', (substr($i, 1), @q);
-                 push @{$self->{'args'}}, $i;
-                 last;
-            }
-            push @{$self->{'args'}}, $i;
-            $ct++;
-        }
+      if ($i =~ /^:/ and $ct) {                # Concatenate :-args.
+        $i = join ' ', (substr($i, 1), @q);
+        push @{$self->{'args'}}, $i;
+        last;
+      }
+      push @{$self->{'args'}}, $i;
+      $ct++;
     }
+  }
 
-    return @{$self->{'args'}};
+  return @{$self->{'args'}};
 }
 
-
 # Dumps the contents of an event to STDERR so you can see what's inside.
 # Takes no args.
 sub dump {
     my ($self, $arg, $counter) = (shift, undef, 0);   # heh heh!
 
-    printf STDERR "TYPE: %-30s    FORMAT: %-30s\n",
-        $self->{'type'}, $self->{'format'};
-    print STDERR "FROM: ", $self->{'from'}, "\n";
-    print STDERR "TO: ", join(", ", @{$self->{'to'}}), "\n";
-    foreach $arg (@{$self->{'args'}}) {
+    printf STDERR "TYPE: %-30s    FORMAT: %-30s\n", $self->type, $self->format;
+    print STDERR "FROM: ", $self->from, "\n";
+    print STDERR "TO: ", join(", ", @{$self->to}), "\n";
+    foreach $arg ($self->args) {
 		print "Arg ", $counter++, ": ", $arg, "\n";
     }
 }
 
-# -- #perl was here! --
-# <Meta-tron> lets say I had [snip]<unknown> this is a lot of text <foo> and I
-#             would like to grab it. <unknown> [/snip] how would I be able to
-#             write a regular expression that matched the stuff between the
-#             <unknown>'s, (the actual contenty of <unknown> is really unknown.
-# <Meta-tron> god Im a mess...
-#      <\mjd> How can you tell the computer to recognize something unknown when
-#             you don't know what it is?
-#      <\mjd> ``Hi, just watch out for something interesting, OK?''
-#    <mendel> with DWIM::Match, of course
-#      <\mjd> ``And pick up a beer if you pass by the 7-11.''
-    
-
 # Sets or returns the format string for this event.
 # Takes 1 optional arg:  the new value for this event's "format" field.
 sub format {
@@ -102,46 +112,30 @@ sub format {
     return $self->{'format'};
 }
 
-# -- #perl was here! --
-# <Mutiny> I'm having this teeny problem and I want to know if somebody
-#          can help me with it..
-#   <\mjd> mutiny: Sever the main neck tendons before cutting through the
-#          spinal cord.  That will allow you more opportunity to separate the
-#          vertebrae prior to removing the head.
-
-
 # Sets or returns the originator of this event
 # Takes 1 optional arg:  the new value for this event's "from" field.
 sub from {
     my $self = shift;
     my @part;
-    
+
     if (@_) {
-	# avoid certain irritating and spurious warnings from this line...
-	{ local $^W;
-	  @part = split /[\@!]/, $_[0], 3;
-        }
+      # avoid certain irritating and spurious warnings from this line...
+      { local $^W;
+        @part = split /[\@!]/, $_[0], 3;
+      }
 	
-	$self->nick(defined $part[0] ? $part[0] : '');
-	$self->user(defined $part[1] ? $part[1] : '');
-	$self->host(defined $part[2] ? $part[2] : '');
-	defined $self->user ?
-	    $self->userhost($self->user . '@' . $self->host) :
-	    $self->userhost($self->host);
-	$self->{'from'} = $_[0];
+      $self->nick(defined $part[0] ? $part[0] : '');
+      $self->user(defined $part[1] ? $part[1] : '');
+      $self->host(defined $part[2] ? $part[2] : '');
+      defined $self->user ?
+          $self->userhost($self->user . '@' . $self->host) :
+          $self->userhost($self->host);
+      $self->{'from'} = $_[0];
     }
+      
     return $self->{'from'};
 }
 
-# -- #perl was here! --
-#    <\mjd>  So, I just heard that some people use their dolls to act out
-#            their childhood traumas.
-#   <jjohn>  \mjd, I've heard of that.
-# <Abigail>  I do that too. Every night before I go to sleep, I whip my dolls.
-#    <\mjd>  Yesterday Lorrie and I had one of our plush octopuses make us
-#            promise that we would never take it to Syms. 
-
-
 # Sets or returns the hostname of this event's initiator
 # Takes 1 optional arg:  the new value for this event's "host" field.
 sub host {
@@ -151,45 +145,6 @@ sub host {
     return $self->{'host'};
 }
 
-# Constructor method for Net::IRC::Event objects.
-# Takes at least 4 args:  the type of event
-#                         the person or server that initiated the event
-#                         the recipient(s) of the event, as arrayref or scalar
-#                         the name of the format string for the event
-#            (optional)   any number of arguments provided by the event
-sub new {
-    my $class = shift;
-
-    # -- #perl was here! --
-    #   \mjd: Under the spreading foreach loop, the lexical variable stands.
-    #   \mjd: The my is a mighty keyword, with abcessed anal glands.
-    #   \mjd: Apologies to Mr. Longfellow.
-
-    my $self = { 'type'   =>  $_[0],
-		 'from'   =>  $_[1],
-		 'to'     =>  ref($_[2]) eq 'ARRAY'  ?  $_[2]  :  [ $_[2] ],
-		 'format' =>  $_[3],
-		 'args'   =>  [ @_[4..$#_] ],
-	       };
-    
-    bless $self, $class;
-    
-    if ($self->{'type'} !~ /\D/) {
-		$self->{'type'} = $self->trans($self->{'type'});
-    } else {
-		$self->{'type'} = lc $self->{'type'};
-    }
-
-    #  ChipDude: "Beware the method call, my son!  The subs that grab, the
-    #             args that shift!"
-    #      \mjd: That's pretty good.
-
-    $self->from($self->{'from'});     # sets nick, user, and host
-    $self->args(@{$self->{'args'}});  # strips colons from args
-    
-    return $self;
-}
-
 # Sets or returns the nick of this event's initiator
 # Takes 1 optional arg:  the new value for this event's "nick" field.
 sub nick {
@@ -199,14 +154,6 @@ sub nick {
     return $self->{'nick'};
 }
 
-# -- #perl was here! --
-#  <ROM_Man>  can anyone point me to a resource on how to deal with shadow
-#             passwords in perl?
-#  <ROM_Man>  anyone alive?
-#     <\mjd>  <rattle> Who dares to disturb my eternal rest?
-#     <\mjd>  <clank>    <clank>                     <clank>
-
-
 # Sets or returns the recipient list for this event
 # Takes any number of args:  this event's list of recipients.
 sub to {
@@ -216,36 +163,6 @@ sub to {
     return wantarray ? @{$self->{'to'}} : $self->{'to'};
 }
 
-# -- #perl was here! --
-#    <\mjd> Last night I dreamt that I had a screaming fight on the telephone
-#           with Sun Microsystems tech sales.
-# <fimmtiu> Seriously?
-#    <\mjd> Seriously.
-#    <crab> \mjd: what were you fighting about?
-#    <\mjd> All sorts of stuff.
-#    <\mjd> They wouldn't deliver what I wanted, they didn't believe I was
-#           affiliated with the people I said I was, they didn't understand
-#           some irregularity in the shipping address,
-#    <\mjd> they wouldn't honor their guarantees...
-#    <\mjd> Finally when I was screaming mad and they were going to have to
-#           give in, they just transferred me to some cheerful marketing droid
-#           who was going to explain the enahancements they'd made to HTML.
-#    <\mjd> So I remember screaming YOU IDIOTS, YOU CAN'T JUST DEFINE
-#           <!--foo--> TO MEAN WHATEVER YOU WANT BECAUSE EVERY BROWSER IN THE
-#           WORLD ALREADY TREATS IT LIKE A COMMENT!
-#           fimmtiu snickers.
-#    <\mjd> That's about when I woke up.
-
-
-# Simple sub for translating server numerics to their appropriate names.
-# Takes one arg:  the number to be translated.
-sub trans {
-    shift if (ref($_[0]) || $_[0]) =~ /^Net::IRC/;
-    my $ev = shift;
-    
-    return (exists $_names{$ev} ? $_names{$ev} : undef);
-}
-
 # Sets or returns the type of this event
 # Takes 1 optional arg:  the new value for this event's "type" field.
 sub type {
@@ -255,12 +172,6 @@ sub type {
     return $self->{'type'};
 }
 
-# -- #perl was here! --
-#    <\mjd>  This is an impressive piece of software.
-# <fimmtiu>  Really? I always thought of it as a huge, monstrously multiplying
-#            collection of hacks in a metaphorical petri dish. :-)
-#    <\mjd>  You say that as though it were a bad thing...
-
 # Sets or returns the username of this event's initiator
 # Takes 1 optional arg:  the new value for this event's "user" field.
 sub user {
@@ -270,22 +181,6 @@ sub user {
     return $self->{'user'};
 }
 
-
-# -- #perl was here! --
-# <Skrewtape> Rap Olympics... heh
-#    <^Pudge> gb, please tell me you are kidding, that there is no such thing.
-#    <Norton> Nope, he's serious
-#      ^Pudge looks for something to smack
-#    <gbacon> i know nothing about it
-#      <\mjd> Who owns the Rap Olympics?
-#    <gbacon> i wonder what the Rap Decathlon is
-#    <^Pudge> i want an ass-kicking olympics.
-#      <\mjd> ``The Rap Olympics belongs to all of humanity.''
-#    <^Pudge> i want to line people up.
-#    <^Pudge> i want them to bend over.
-#    <^Pudge> and i want to kick their asses and get points and medals for it.
-
-
 # Just $self->user plus '@' plus $self->host, for convenience.
 sub userhost {
     my $self = shift;
@@ -294,7 +189,14 @@ sub userhost {
     return $self->{'userhost'};
 }
 
-
+# Simple sub for translating server numerics to their appropriate names.
+# Takes one arg:  the number to be translated.
+sub trans {
+    shift if (ref($_[0]) || $_[0]) =~ /^Net::IRC/;
+    my $ev = shift;
+    
+    return (exists $_names{$ev} ? $_names{$ev} : undef);
+}
 
 %_names = (
 	   # suck!  these aren't treated as strings --
@@ -328,7 +230,14 @@ sub userhost {
 	   217 => "statsqline",
 	   218 => "statsyline",
 	   219 => "endofstats",
+           220 => "statsbline",             # UnrealIrcd, Hendrik Frenzel
 	   221 => "umodeis",
+           222 => "sqline_nick",            # UnrealIrcd, Hendrik Frenzel
+           223 => "statsgline",             # UnrealIrcd, Hendrik Frenzel
+           224 => "statstline",             # UnrealIrcd, Hendrik Frenzel
+           225 => "statseline",             # UnrealIrcd, Hendrik Frenzel
+           226 => "statsnline",             # UnrealIrcd, Hendrik Frenzel
+           227 => "statsvline",             # UnrealIrcd, Hendrik Frenzel
 	   231 => "serviceinfo",
 	   232 => "endofservices",
 	   233 => "service",
@@ -341,12 +250,10 @@ sub userhost {
 	   245 => "statssline",		# Reserved, Kajetan@Hinner.com, 17/10/98
 	   246 => "statstline",		# Undernet Extension, Kajetan@Hinner.com, 17/10/98
 	   247 => "statsgline",		# Undernet Extension, Kajetan@Hinner.com, 17/10/98
+### TODO: need numerics to be able to map to multiple strings
+###           247 => "statsxline",             # UnrealIrcd, Hendrik Frenzel
 	   248 => "statsuline",		# Undernet Extension, Kajetan@Hinner.com, 17/10/98
 	   249 => "statsdebug",		# Unspecific Extension, Kajetan@Hinner.com, 17/10/98
-	   250 => "statsconn",		# Undernet Extension, Kajetan@Hinner.com, 17/10/98
-	   
-	  
-
 	   250 => "luserconns",   # 1998-03-15 -- tkil
 	   251 => "luserclient",
 	   252 => "luserop",
@@ -366,8 +273,13 @@ sub userhost {
 	   275 => "statsdline",		# Undernet Extension, Kajetan@Hinner.com, 17/10/98
 	   280 => "glist",		# Undernet Extension, Kajetan@Hinner.com, 17/10/98
 	   281 => "endofglist",		# Undernet Extension, Kajetan@Hinner.com, 17/10/98
+           290 => "helphdr",            # UnrealIrcd, Hendrik Frenzel
+           291 => "helpop",             # UnrealIrcd, Hendrik Frenzel
+           292 => "helptlr",            # UnrealIrcd, Hendrik Frenzel
+           293 => "helphlp",            # UnrealIrcd, Hendrik Frenzel
+           294 => "helpfwd",            # UnrealIrcd, Hendrik Frenzel
+           295 => "helpign",            # UnrealIrcd, Hendrik Frenzel
 
-	   
 	   300 => "none",
 	   301 => "away",
 	   302 => "userhost",
@@ -376,6 +288,8 @@ sub userhost {
 	   305 => "unaway",
 	   306 => "nowaway",
 	   307 => "userip",		# Undernet Extension, Kajetan@Hinner.com, 17/10/98
+           308 => "rulesstart",         # UnrealIrcd, Hendrik Frenzel
+           309 => "endofrules",         # UnrealIrcd, Hendrik Frenzel
 	   310 => "whoishelp",          # (July01-01)Austnet Extension, found by Andypoo <andypoo@secret.com.au>
 	   311 => "whoisuser",
 	   312 => "whoisserver",
@@ -396,8 +310,13 @@ sub userhost {
 	   332 => "topic",
 	   333 => "topicinfo",      # 1997-11-24 -- archon
 	   334 => "listusage",		# Undernet Extension, Kajetan@Hinner.com, 17/10/98
+           335 => "whoisbot",           # UnrealIrcd, Hendrik Frenzel
 	   341 => "inviting",
 	   342 => "summoning",
+           346 => "invitelist",         # UnrealIrcd, Hendrik Frenzel
+           347 => "endofinvitelist",    # UnrealIrcd, Hendrik Frenzel
+           348 => "exlist",             # UnrealIrcd, Hendrik Frenzel
+           349 => "endofexlist",        # UnrealIrcd, Hendrik Frenzel
 	   351 => "version",
 	   352 => "whoreply",
 	   353 => "namreply",
@@ -419,16 +338,22 @@ sub userhost {
 	   376 => "endofmotd",
 	   377 => "motd2",        # 1997-10-16 -- tkil
 	   378 => "austmotd",		# (July01-01)Austnet Extension, found by Andypoo <andypoo@secret.com.au>
+           379 => "whoismodes",         # UnrealIrcd, Hendrik Frenzel
 	   381 => "youreoper",
 	   382 => "rehashing",
+           383 => "youreservice",       # UnrealIrcd, Hendrik Frenzel
 	   384 => "myportis",
 	   385 => "notoperanymore",	# Unspecific Extension, Kajetan@Hinner.com, 17/10/98
+           386 => "qlist",              # UnrealIrcd, Hendrik Frenzel
+           387 => "endofqlist",         # UnrealIrcd, Hendrik Frenzel
+           388 => "alist",              # UnrealIrcd, Hendrik Frenzel
+           389 => "endofalist",         # UnrealIrcd, Hendrik Frenzel
 	   391 => "time",
 	   392 => "usersstart",
 	   393 => "users",
 	   394 => "endofusers",
 	   395 => "nousers",
-	   
+
 	   401 => "nosuchnick",
 	   402 => "nosuchserver",
 	   403 => "nosuchchannel",
@@ -436,6 +361,7 @@ sub userhost {
 	   405 => "toomanychannels",
 	   406 => "wasnosuchnick",
 	   407 => "toomanytargets",
+           408 => "nosuchservice",      # UnrealIrcd, Hendrik Frenzel
 	   409 => "noorigin",
 	   411 => "norecipient",
 	   412 => "notexttosend",
@@ -446,21 +372,28 @@ sub userhost {
 	   422 => "nomotd",
 	   423 => "noadmininfo",
 	   424 => "fileerror",
+           425 => "noopermotd",         # UnrealIrcd, Hendrik Frenzel
 	   431 => "nonicknamegiven",
 	   432 => "erroneusnickname",   # This iz how its speld in thee RFC.
 	   433 => "nicknameinuse",
+           434 => "norules",            # UnrealIrcd, Hendrik Frenzel
+           435 => "serviceconfused",    # UnrealIrcd, Hendrik Frenzel
 	   436 => "nickcollision",
 	   437 => "bannickchange",		# Undernet Extension, Kajetan@Hinner.com, 17/10/98
 	   438 => "nicktoofast",		# Undernet Extension, Kajetan@Hinner.com, 17/10/98
 	   439 => "targettoofast",		# Undernet Extension, Kajetan@Hinner.com, 17/10/98
-           304 => "servicesdown",           # Bahamut IRCD
+           440 => "servicesdown",           # Bahamut IRCD
 	   441 => "usernotinchannel",
 	   442 => "notonchannel",
 	   443 => "useronchannel",
 	   444 => "nologin",
 	   445 => "summondisabled",
 	   446 => "usersdisabled",
+           447 => "nonickchange",       # UnrealIrcd, Hendrik Frenzel
 	   451 => "notregistered",
+           455 => "hostilename",        # UnrealIrcd, Hendrik Frenzel
+           459 => "nohiding",           # UnrealIrcd, Hendrik Frenzel
+           460 => "notforhalfops",      # UnrealIrcd, Hendrik Frenzel
 	   461 => "needmoreparams",
 	   462 => "alreadyregistered",
 	   463 => "nopermforhost",
@@ -469,6 +402,8 @@ sub userhost {
 	   466 => "youwillbebanned",
 	   467 => "keyset",
 	   468 => "invalidusername",		# Undernet Extension, Kajetan@Hinner.com, 17/10/98
+           469 => "linkset",            # UnrealIrcd, Hendrik Frenzel
+           470 => "linkchannel",        # UnrealIrcd, Hendrik Frenzel
 	   471 => "channelisfull",
 	   472 => "unknownmode",
 	   473 => "inviteonlychan",
@@ -478,23 +413,44 @@ sub userhost {
            477 => "needreggednick",           # Bahamut IRCD
 	   478 => "banlistfull",		# Undernet Extension, Kajetan@Hinner.com, 17/10/98
            479 => "secureonlychannel",        # pircd
+### TODO: see above todo
+###           479 => "linkfail",               # UnrealIrcd, Hendrik Frenzel
+           480 => "cannotknock",        # UnrealIrcd, Hendrik Frenzel
 	   481 => "noprivileges",
 	   482 => "chanoprivsneeded",
 	   483 => "cantkillserver",
 	   484 => "ischanservice",		# Undernet Extension, Kajetan@Hinner.com, 17/10/98
+           485 => "killdeny",           # UnrealIrcd, Hendrik Frenzel
+           486 => "htmdisabled",        # UnrealIrcd, Hendrik Frenzel
+           489 => "secureonlychan",     # UnrealIrcd, Hendrik Frenzel
 	   491 => "nooperhost",
 	   492 => "noservicehost",
-	   
+
 	   501 => "umodeunknownflag",
 	   502 => "usersdontmatch",
 
 	   511 => "silelistfull",		# Undernet Extension, Kajetan@Hinner.com, 17/10/98
 	   513 => "nosuchgline",		# Undernet Extension, Kajetan@Hinner.com, 17/10/98
 	   513 => "badping",			# Undernet Extension, Kajetan@Hinner.com, 17/10/98
+           518 => "noinvite",           # UnrealIrcd, Hendrik Frenzel
+           519 => "admonly",            # UnrealIrcd, Hendrik Frenzel
+           520 => "operonly",           # UnrealIrcd, Hendrik Frenzel
+           521 => "listsyntax",         # UnrealIrcd, Hendrik Frenzel
+           524 => "operspverify",       # UnrealIrcd, Hendrik Frenzel
+
            600 => "rpl_logon",           # Bahamut IRCD
            601 => "rpl_logoff",           # Bahamut IRCD
+           602 => "rpl_watchoff",       # UnrealIrcd, Hendrik Frenzel
+           603 => "rpl_watchstat",      # UnrealIrcd, Hendrik Frenzel
            604 => "rpl_nowon",           # Bahamut IRCD
            605 => "rpl_nowoff",           # Bahamut IRCD
+           606 => "rpl_watchlist",      # UnrealIrcd, Hendrik Frenzel
+           607 => "rpl_endofwatchlist", # UnrealIrcd, Hendrik Frenzel
+           610 => "mapmore",            # UnrealIrcd, Hendrik Frenzel
+           640 => "rpl_dumping",        # UnrealIrcd, Hendrik Frenzel
+           641 => "rpl_dumprpl",        # UnrealIrcd, Hendrik Frenzel
+           642 => "rpl_eodump",         # UnrealIrcd, Hendrik Frenzel
+
            999 => "numericerror",           # Bahamut IRCD
 
 	  );
